'use strict';

var SetupOTP = function()
{
	var setup = {};
	
	setup.initPage = function()
	{
		Events.addEventHandler('username', 'change', setup.getQrCode);
		Events.addEventHandler('otp', 'keypress', Events.integerOnly);

		Events.addEventHandler('username', 'keypress', setup.focusOnEnter);
		Events.addEventHandler('password', 'keypress', setup.focusOnEnter);
		Events.addEventHandler('newpw', 'keypress', setup.focusOnEnter);
		Events.addEventHandler('newpw2', 'keypress', setup.focusOnEnter);
		Events.doOnEnter('otp', setup.save);
		
		Events.addEventHandler('change-pwd', 'change', setup.enableChangePassword);
		
		Buttons.addClickHandler('btnCancel', App.closeWindow);
		Buttons.addClickHandler('btnClose', App.closeWindow);
		Buttons.addClickHandler('btnSave', setup.save);
		
		var username = Doc.element('username').value;
		
		if (username)
		{
			Events.sendChangeEvent('username');
			Doc.element('password').focus();
		}
		else
		{
			Doc.element('username').focus();
		}
	}
		
	setup.getQrCode = function()
	{
		var username = Doc.element('username').value;
		if (username == '') 
		{
			Doc.show('no-qrcode', 'inline-block');
			Doc.hide('qrcode');
			return;
		}
		
		var url = 'OTP.servlet?ID='+App.getSessionId();
		url += '&cmd=CreateQRCode&username='+encodeURIComponent(username);

		var request = Ajax.getXmlHttpRequest();
		request.open('GET', url, true);
		Ajax.setReadyStateHandler(request, setup.qrCodeLoaded, setup.qrCodeLoaded);
		request.send(null);
	}

	setup.getResponse = function(request)
	{
		var response;
		var error = false;
		try
		{
			response = JSON.parse(request.responseText);
			if (response.status != 'ok')
			{
				error = response.message || response.stacktrace || 'Unexpected error';
			}
		}
		catch (ex)
		{
			error = ex;
		}
		
		if (error)
		{
			setup.showError(Strings.encodeTags(error));
			return;
		}
		
		return response;
	}
	
	setup.qrCodeLoaded = function(request)
	{
		var response = setup.getResponse(request);
		if (!response) return;
		
		Doc.element('qrcode').src = 'data:image/png;base64,'+response.image;
		Doc.show('qrcode', 'inline');
		Doc.hide('no-qrcode');
	}

	setup.enableChangePassword = function()
	{
		Doc.hide('change-password');
		Doc.show('new-password-1');
		Doc.show('new-password-2');
		Doc.element('newpw').focus();
	}
	
	setup.showError = function(message, focusElement)
	{
		Doc.element('message').innerHTML = '<div class="error">'+message+'</div>';
		if (focusElement) Doc.element(focusElement).focus();
	}
	
	setup.focusOnEnter = function(event)
	{
		if (event.keyCode == 13)
		{
			var nextFocus = Doc.element(Data.get(event.currentTarget, 'next-focus'));
			if (nextFocus && nextFocus.focus) nextFocus.focus();
		}
	}
	
	setup.save = function()
	{
		var username = Doc.element('username').value;
		var password = Doc.element('password').value;
		var otp = Doc.element('otp').value;
		var newPw = Doc.element('newpw').value;
		var newPw2 = Doc.element('newpw2').value;
		
		if (username == '')
		{
			setup.showError('A username must be specified', 'username');
			return;
		}
		else if (password == '')
		{
			setup.showError('A password must be specified', 'password');
			return;
		}
		else if (otp == '')
		{
			setup.showError('A one-time passcode must be specified', 'otp');
			return;
		}
		else if (newPw != newPw2)
		{
			setup.showError('The new passwords do not match', 'newpw2');
			return;
		}
		
		var url = 'OTP.servlet?ID='+App.getSessionId();
		url += '&cmd=SaveOTPSetup';
		
		var postData = {};
		postData.username = username;
		postData.password = password;
		postData.otp = otp;
		if (newPw)
		{
			postData.newPassword = newPw;
		}
		
		var request = Ajax.getXmlHttpRequest();
		request.open('POST', url, true);
		request.setRequestHeader("Content-Type", "application/json; charset=UTF-8");
		Ajax.setReadyStateHandler(request, setup.setupCompleted, setup.setupCompleted);
		request.send(JSON.stringify(postData));
	}
	
	setup.setupCompleted = function(request)
	{
		var response = setup.getResponse(request);
		if (!response) return;
		
		Doc.element('message').innerHTML = '<div class="success bg-filled-100">'+Strings.encodeTags(response.message)+'</div>';
		
		Doc.hide('setup-buttons');
		Doc.hide('setup-form');
		Doc.show('close-button');
	}
	
	return setup;
}();

Doc.onLoad(SetupOTP.initPage);

